/**********************************************************************************************************/
/* Replication program for primary data construction "FKR2024_CrowdingIn.dta" for ""Crowding in" Effect of Public Investment on Private Investment Revisited"    */
/* Authored by: John Nana Francois (World Bank), Maty Konte (IFC), and Franz U Ruch (World Bank)     */
/* Contact: jfrancois1@worldbank.org; mkonte@ifc.org and fruch@worldbank.org */ 																										
/***********************************************************************************************************/

#delimit ;
clear;clear matrix;
set more off;
set matsize 800;



/* Import WEO gdp: WEO_Data contains the original downloaded WEO GDP Data*/
import excel "${data}\WEO_Data.xlsx", sheet("Stata_input") firstrow;
generate ctry = iso;
encode iso, generate(iso1);
encode country, generate(country1);
drop iso country;
rename iso1 iso; 
rename country1 country;


/*** Transform WEO Data into Panel */
reshape long y , i(iso Var ) j(year);
reshape wide y , i(iso year ) j(Var) string;
order country iso;

gen weogdpcd = yweogdpcd*1000000000; 
gen weogdpcl = yweogdpcl*1000000000; 
gen weogdpkl = yweogdpkl*1000000000; 

drop country iso;

/*Save WEO GDP Panel Data */
save "${data}\weo_gdp.dta", replace;

clear;
/*Import IMF Private and Public Investment Data */
import excel "${data}\IMFInvestmentandCapitalStockDataset2021.xlsx", sheet("Dataset") firstrow;
rename isocode ctry;
/*Save IMF Private and Public Investment Data */
save "${data}\investment_imf.dta", replace;

clear; 

/*Import and save WB income classification */
import excel "${data}\wdi_income_classifications_2024.xlsx", sheet("wdi_income_groups") firstrow;
save"${data}\wdi_income_groups", replace;

clear; 

/*Import and save WB IDA countries  */
import excel "${data}\ida_wdi_2024.xlsx", sheet("ida") firstrow;
save "${data}\ida_wdi_2024.dta", replace;

clear;

/*Import WGI dataset*/

use "${data}\wgidataset.dta", clear;
keep code year cce;
rename code ctry;

save "${data}\wgi_cce.dta", replace;

/*Import FD dataset*/
use "${data}\wgidataset.dta", clear;
keep code year cce;
rename code ctry;

save "${data}\wgi_cce.dta", replace;

/** Merge to construct final raw data for the analysis ***/
use "${data}\investment_imf.dta", clear;

merge m:1 ctry using "${data}\wdi_income_groups.dta", keep(match);
drop _merge;

merge m:1 year ctry using "${data}\weo_gdp.dta", keep(match);
drop _merge;
 
/*merge m:1 year ctry using ivdata_ulli_v0.dta, keep(match);*/
merge m:1 year ctry using "${data}\Aggregated_DRS.dta", keep(match);
drop _merge;

merge m:1 year ctry using "${data}\wgi_cce.dta";
/*keep if _merge==3;*/
drop _merge;

merge m:1 year ctry using "${data}\FD_Database_STATA.dta";
drop if _merge==2;
drop _merge;

merge m:1 ctry year using "${data}\kaopen_2021.dta";
drop if _merge==2;
drop _merge;
 
merge m:1  ctry using "${data}\ida_wdi_2024.dta";
drop _merge;

merge m:1 ctry using "${data}\region_dummy.dta";
drop _merge;

egen id = group(ctry);
gen ida= 0; replace ida=1 if ida_wdi=="ida";

tsset id year;

/* Final Data For Main Regression Analysis */
save "${data}\FKR2024_CrowdingIn.dta", replace;


